﻿/**
 * Wrapper class for the origo API based on the jsolait XML-RPC module (http://jsolait.net/wiki/documentation/xmlrpc).
 */
var origorpc =
{
    appKey:  "",
    userKey: "",
    session: "",
    service: null,

    init: function()
	{
	    // for firefox
	    //netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
	
	    var xmlrpc=null;
        try {
            xmlrpc = imprt("xmlrpc");
        }
        catch(e) {
            this.reportException(e);
            throw "importing of xmlrpc module failed.";
        }
    	
	    var addr    = "http://api.origo.ethz.ch/api/xmlrpc";
	    var methods = ["user.login_key", "project.list_of_user", "user.my_name",
	                   "workitem.retrieve", "workitem.list", "workitem.list_projects", "workitem.list_project", "workitem.set_read_status",
	                   "project.add", "project.request_add", "project.request_retrieve", "project.list", "project.list_of_user", "project.members", "project.list_bookmarkers", "project.change_group", "project.retrieve", "project.change_description", "project.change_logo"];
	                   
	    this.service = new xmlrpc.ServiceProxy(addr, methods);
	},
	
	handleException: function(e)
	{
	    var err = new OrigoRPCError();	    
	
	    if (typeof e == "string")
	    {
	        err.faultString = e;
	    }
	    else if ((e instanceof Object) && (e.faultString != null))
	    {
	        err.faultString = e.faultString;
	    }
	    else if ((e instanceof Object) && (e.name != null) && (e.message != null))
	    {
	        err.faultName   = "SendFailed";
	        err.faultString = e.name +": "+ e.message;
	    }
	    else
	    {
	        err.faultName   = "unkown";
	        err.faultString = "unknown error";
	    }
	    
	    this.onError(err);
	},
	
	/**
	 * default implementation
	 * Overwrite this too get an Error string and display it
	 **/
	onError: function(err) {},

    /**
     * Check if exception was due to "session not valid", iff yes login in and execute
     * func(args[])
     *
     * e: Exception
     * func: function to be executed if login succeeds
     * args: argument of func wrapped into an array
     *
     * Example:
     *    retyException(e, origorpc.workitem.retrieve, [12345]);
     */
    retryException: function(e, func, args, fromExceptionHandler)
    {
        try {
            //if (e.faultString.toLowerCase().indexOf("session not valid") != -1)
            if (!fromExceptionHandler) {
                if (!origorpc.login()) {
                    throw "Error: Could not renew session";
                }
                else {
                    args.push(true);
                    return func.apply(this,args);
                }
            }
            else {
                throw "Error: unknow exception occured";
            }
        }
        catch (e) {
            this.handleException(e);
        }
    },

    login: function()
	{
	    try {
	        var loginOk = false;
	        if (this.userKey != "") {
		        try {
			        this.session = this.service.user.login_key(this.userKey, this.appKey);
			        loginOk = true;
		        } catch (e) {
		            this.handleException(e);
		        }
	        }
	        else {
	            throw "There is no user key set.";
	        }
	    }
        catch (e) {
            this.handleException(e);
        }
    	
	    return loginOk;
    },
    
    user:
    {
        /**
         * Your user name.
         */
        my_name: function() //: String
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        return origorpc.service.user.my_name(origorpc.session);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.user.my_name, [], fromExceptionHandler);
            }
        }
    },
    
    project:
    {
        /**
         * Create a new project.
         * (needs project_create permission)
         */
        add: function(project_name, description, is_closed_source, is_hidden, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        return origorpc.service.project.add(origorpc.session, description, is_closed_source, is_hidden);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.add, [description, is_closed_source, is_hidden], fromExceptionHandler);
            }
        },

        /**
         * Request creation of a new project. Projects are manually created by the Origo Team.
         */
        request_add: function(project_name, description, is_closed_source, message_to_origo_team, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        return origorpc.service.project.request_add(origorpc.session, project_name, description, is_closed_source, message_to_origo_team);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.request_add, [project_name, description, is_closed_source, message_to_origo_team], fromExceptionHandler);
            }
        },

        /**
         * Retrieve a project-request.
         */
        request_retrieve: function(project_name, fromExceptionHandler) //: project_struct
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        return origorpc.service.project.request_retrieve(origorpc.session, project_name, fromExceptionHandler);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.request_retrieve, [project_name, fromExceptionHandler], fromExceptionHandler);
            }
        },

       /**
        * List all available projects (needs project_list permission). Hidden
        * projects are not listed in the usual project lists but can be explicitly
        * added to the result by setting show_hidden to true.
        */
       list: function(show_hidden, fromExceptionHandler) //: array[struct[project_id, name]]
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        return origorpc.service.project.list(origorpc.session, show_hidden);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.list, [show_hidden], fromExceptionHandler);
            }
        },

        /**
         * Lists all projects of which 'user_name' is member or owner.
         * group_id 3 is a project owner and 4 is project member.
         * (needs project_list_of_user permission)
         */
        list_of_user: function(user_name, fromExceptionHandler) //: array[struct[project_id, project_name, group_id]]
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        return origorpc.service.project.list_of_user(origorpc.session, user_name);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.list_of_user, [user_name], fromExceptionHandler);
            }
        },

        /**
         * List project members with a certain access group.
         * access_group 3 is project owner and 4 is project member.
         * (needs project_members_list permission)
         */
        members: function(project_id, access_group, fromExceptionHandler) //: array[struct[name]]
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        var id = parseInt(project_id);
		        return origorpc.service.project.members(origorpc.session, id, access_group);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.members, [project_id, access_group], fromExceptionHandler);
            }
        },

        /**
         * List project bookmarkers.
         * (needs project_members_list permission)
         */
        list_bookmarkers: function(project_id, fromExceptionHandler) //: array[struct[name]]
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        var id = parseInt(project_id);
		        return origorpc.service.project.list_bookmarkers(origorpc.session, id);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.list_bookmarkers, [project_id], fromExceptionHandler);
            }
        },

        /**
         * Change access group of 'user_name' in project to 'access_group'
         * (0=no rights, 3=project owner, 4=project member). If the user
         * becomes owner or member all workitems will be subscribed except
         * the user already bookmarked the project. If the user becomes 0
         * all workitem subscriptions will be removed except the user has*
         * still bookmarked the project.
         * (needs project_change_group permission)
         */
        change_group: function(project_id, user_name, access_group, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        var id = parseInt(project_id);
		        return origorpc.service.project.change_group(origorpc.session, id, user_name, access_group);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.change_group, [project_id, user_name, access_group], fromExceptionHandler);
            }
        },

        /**
         * Retrieve a project.
         */
        retrieve: function(project_id, fromExceptionHandler)//: struct[project_id, name, description, logo, is_closed_source]
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        var id = parseInt(project_id);
		        return origorpc.service.project.retrieve(origorpc.session, id);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.retrieve, [project_id], fromExceptionHandler);
            }
        },

        /**
         * Change the description of project 'project_id'.
         */
        change_description: function(project_id, description, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        var id = parseInt(project_id);
		        return origorpc.service.project.change_description(origorpc.session, id, description);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.change_description, [project_id, description], fromExceptionHandler);
            }
        },

        /**
         * Change the logo of a project. The logo has to be uploaded via FTP first.
         */
        change_logo: function(project_id, img_file_name, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        var id = parseInt(project_id);
		        return origorpc.service.project.change_logo(origorpc.session, id, img_file_name);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.project.change_logo, [project_id, img_file_name], fromExceptionHandler);
            }
        }
    },
    
    workitem:
    {    
        retrieve: function(workitem_id, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;
        
            try {
		        var id = parseInt(workitem_id);
		        return origorpc.service.workitem.retrieve(origorpc.session, id);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.workitem.retrieve, [workitem_id], fromExceptionHandler);
            }
        },
        
        list: function(limit, unread_only, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;

            try {
		        var iLimit = parseInt(limit);
		        return origorpc.service.workitem.retrieve(origorpc.session, iLimit, unread_only);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.workitem.list, [limit, unread_only], fromExceptionHandler);
            }
        },
        
        list_projects: function(project_id, limit, unread_only, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;

            try {
                var id = parseInt(project_id);
		        var iLimit = parseInt(limit);
		        return origorpc.service.workitem.retrieve(origorpc.session, id, iLimit, unread_only);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.workitem.list_projects, [project_id, limit, unread_only], fromExceptionHandler);
            }
        },
        
        list_project: function(project_id, limit, unread_only, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;

            try {
		        var id = parseInt(project_id);
		        return origorpc.service.workitem.list_project(origorpc.session, id, limit, unread_only);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.workitem.list_project, [project_id, limit, unread_only], fromExceptionHandler);
	        }
        },
        
        set_read_status: function(session, workitem_id, read_status, fromExceptionHandler)
        {
            fromExceptionHandler = (typeof(fromExceptionHandler) != 'undefined') ? fromExceptionHandler : false;

            try {
		        var id = parseInt(workitem_id);
		        return origorpc.service.workitem.list_project(origorpc.session, id, read_status);
	        }
	        catch (e) {
	            return origorpc.retryException(e, origorpc.workitem.set_read_status, [workitem_id, read_status], fromExceptionHandler);
	        }
        }
    } 
}

// Returns a string representation of a unix timestamp as it is used in origo for creation dates
function getDateString(creationTime) {
	// IMPORTANT: Unix timestamps are seconds but Date constructor takes miliseconds
	var date = new Date(creationTime*1000);
	var hours = date.getHours() + "";
	var minutes = date.getMinutes() + "";
	var seconds = date.getSeconds() + "";
	var month = date.getMonth()+1;
	
	if (hours.length == 1) {
		hours = "0" + hours;
	}
	if (minutes.length == 1) {
		minutes = "0" + minutes;
	}
	if (seconds.length == 1) {
		seconds = "0" + seconds;
	}

	return date.getFullYear() + "/" + month + "/" + date.getDate() + " " + hours + ":" + minutes + ":" + seconds;
}

/**
 * OrigoRPCError - error container
 */
 function OrigoRPCError() {
    this.faultString = "";
    this.faultName   = "";
    this.base        = null;
}

OrigoRPCError.prototype.faultString;
OrigoRPCError.prototype.faultName;
OrigoRPCError.prototype.base;

/**
 * Debug class - error container
 * Todo: make it OS-Independent
 */
var debug = {
    /**
     * logLevels:
     * - 0:   nothing
     * - 1:   errors
     * - 2:   warning
     * - 3-5: info
     * - 6-8: debug 
     */
    logLevel: 2,
    
    filename: "log.txt",
    
    /**
     * LogLevels constants
     */
    level: {
        error:   1,
        warning: 2,
        info1:   3,
        info2:   4,
        info3:   5,
        debug1:  6,
        debug2:  7,
        debug3:  8
    },
    
    /**
     * private
     */
    fso: null,
    file: null,
    
    init: function() {
        this.fso = new ActiveXObject("Scripting.FileSystemObject");
    },
    
    log: function(level, msg) {
        if (level <= this.logLevel) {
            var path = System.Gadget.path + "/../"
            this.file = this.fso.OpenTextFile(path + this.filename, 8 /*ForAppending*/, true);
            
            this.file.WriteLine(msg)
            this.file.Close();
        }
    },
    
    print_r: function(input, _indent) {
        if(typeof(_indent) == 'string') {
            var indent = _indent + ' ';
            var paren_indent = _indent + ' ';
        }
        else {
            var indent = ' ';
            var paren_indent = '';
        }
        
        switch(typeof(input)) {
            case 'boolean':
                var output = (input ? 'true' : 'false') + "\n";
                break;
            case 'object':
                if ( input===null ) {
                    var output = "null\n";
                    break;
                }
        
                var output = ((input.reverse) ? 'Array' : 'Object') + " (\n";
                
                for(var i in input) {
                    output += indent + "[" + i + "] => " + this.print_r(input[i], indent);
                }
            
                output += paren_indent + ")\n";
                break;
            case 'number':
            case 'string':
            default:
                var output = "" + input + "\n";
        }
        return output;
    }
}

debug.init();
debug.filename = "origolog.txt";

